import mimetypes
import time

from django.conf import settings
from django.db.models.functions import Now
from django.http.response import FileResponse, HttpResponseNotModified
from django.shortcuts import get_object_or_404
from django.utils.http import http_date
from django.views.generic.base import View
from django.views.static import was_modified_since

from dbstorage.models import DBFile


class DBFileView(View):
    def get(self, request, name):
        """Endpoint to return memory storage file"""
        db_file_query = DBFile.objects.defer("content").filter(name=name)
        db_file = get_object_or_404(db_file_query)
        if getattr(settings, "DJANGO_DBFILE_TRACK_ACCESSED", False):
            db_file_query.update(accessed_on=Now())

        mtime = time.mktime(db_file.updated_on.timetuple())
        modified = was_modified_since(
            header=self.request.META.get("HTTP_IF_MODIFIED_SINCE"),
            mtime=mtime,
        )

        if not modified:
            return HttpResponseNotModified()

        content_type, encoding = mimetypes.guess_type(db_file.name)
        content_type = content_type or "application/octet-stream"

        response = FileResponse(db_file.content, content_type=content_type)
        response["Last-Modified"] = http_date(mtime)
        response["Content-Length"] = db_file.size
        if encoding:
            response["Content-Encoding"] = encoding
        return response



from urllib.parse import urljoin

from django.conf import settings
from django.core.files.base import ContentFile
from django.core.files.storage import Storage
from django.utils.encoding import filepath_to_uri

from dbstorage.models import DBFile


class DBStorage(Storage):
    def __init__(self, base_url=None):
        if base_url is None:
            base_url = settings.MEDIA_URL
        if not base_url.endswith("/"):
            base_url += "/"
        self.base_url = base_url

    def open(self, name, mode="rb"):
        return self._open(name, mode)

    def _open(self, name, mode="rb"):
        f = DBFile.objects.only("content").get(name=name)
        return ContentFile(f.content)

    def _save(self, name, content):
        name = self.get_available_name(name, max_length=255)
        DBFile.objects.create(content=content.read(), name=name)
        return name

    def get_valid_name(self, name):
        return name[:255]

    def path(self, name):
        raise NotImplementedError("DBStorage does not support path() method")

    def delete(self, name):
        DBFile.objects.filter(name=name).delete()

    def exists(self, name):
        return DBFile.objects.filter(name=name).exists()

    def listdir(self, path):
        raise NotImplementedError(
            "DBStorage does not support listdir() method"
        )

    def size(self, name):
        return DBFile.objects.only().get(name=name).size

    def url(self, name):
        return urljoin(self.base_url, filepath_to_uri(name))

    def accessed_time(self, name):
        return DBFile.objects.only("accessed_on").get(name=name).accessed_on

    def created_time(self, name):
        return DBFile.objects.only("created_on").get(name=name).created_on

    def modified_time(self, name):
        return DBFile.objects.only("updated_on").get(name=name).updated_on



# -*- coding: utf-8 -*-
# Generated by Django 1.9.2 on 2016-02-24 23:31
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='DBFile',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('content', models.BinaryField()),
                ('name', models.CharField(max_length=255, unique=True)),
                ('size', models.IntegerField(default=0)),
                ('created_on', models.DateTimeField(auto_now_add=True)),
                ('updated_on', models.DateTimeField(auto_now=True)),
            ],
            options={
                'verbose_name': 'DB file',
                'db_table': 'db_file',
            },
        ),
    ]



from django.db import models
from django.utils import timezone
from django.db.models.functions import Length
from django.conf import settings


class DBFileManager(models.Manager):
    def get_queryset(self):
        return super().get_queryset().annotate(size=Length("content"))


class DBFile(models.Model):
    content = models.BinaryField(editable=False)
    name = models.CharField(max_length=255, unique=True)
    created_on = models.DateTimeField(auto_now_add=True)
    updated_on = models.DateTimeField(auto_now=True)
    accessed_on = models.DateTimeField(default=timezone.now)

    objects = DBFileManager()
    if getattr(settings, "DJANGO_DBFILE_DB", "default") != "default":
        objects = objects.db_manager(getattr(settings, "DJANGO_DBFILE_DB"))

    # size is now a virtual field

    class Meta:
        db_table = "db_file"
        verbose_name = "DB file"

    def __str__(self):
        return self.name



from django import forms
from django.urls import reverse
from django.utils.safestring import mark_safe

from dbstorage.models import DBFile

widget_template = """\
<table>
    <tbody>
        <tr>
            <td>Current File</td>
            <td><a href="{url}" target="_blank">{name}</a></td>
        </tr>
        <tr class="alt">
            <td style="vertical-align: middle">Replace File</td>
            <td>{input}</td>
        </tr>
    </tbody>
</table>
"""


class DBFileWidget(forms.FileInput):
    def render(self, name, value, attrs=None, renderer=None):
        input_html = super(DBFileWidget, self).render(
            name, value, attrs, renderer
        )
        if not self.instance:
            return input_html

        context = {
            "input": input_html,
            "name": self.instance.name,
            "url": reverse("db_file", args=[self.instance.name]),
        }
        return mark_safe(widget_template.format(**context))


class DBFileForm(forms.ModelForm):
    file = forms.FileField(widget=DBFileWidget())

    class Meta:
        model = DBFile
        fields = ["name"]

    def __init__(self, *args, **kwargs):
        super(DBFileForm, self).__init__(*args, **kwargs)

        instance = kwargs.get("instance")
        field = self.fields["file"]
        field.widget.instance = instance
        if instance:
            field.required = False

    def save(self, commit=True):
        db_file = super(DBFileForm, self).save(commit=False)
        file = self.cleaned_data.get("file")
        if file:
            db_file.content = file.read()
        if commit:
            db_file.save()
        return db_file



import re

from django.conf import settings
from django.urls import re_path

from dbstorage import views


urlpatterns = [
    re_path(
        r"^%s(?P<name>.*)$" % re.escape(settings.MEDIA_URL.lstrip("/")),
        views.DBFileView.as_view(),
        name="db_file",
    ),
]



from django.contrib import admin
from django.template.defaultfilters import filesizeformat
from django.urls import reverse
from django.utils.html import mark_safe

from dbstorage.forms import DBFileForm
from dbstorage.models import DBFile


@admin.register(DBFile)
class DBFileAdmin(admin.ModelAdmin):
    form = DBFileForm
    list_display = [
        "name",
        "download",
        "file_size",
        "updated_on",
        "created_on",
    ]
    ordering = ["name"]
    search_fields = ["name"]

    def download(self, obj):
        href = reverse("db_file", args=[obj.name])
        return mark_safe(
            '<a href="{}" target="_blank">Download</a>'.format(href)
        )

    download.allow_tags = True
    download.short_description = ""

    def file_size(self, obj):
        return filesizeformat(obj.size)

    file_size.short_description = "size"

    def get_queryset(self, request):
        qs = super(DBFileAdmin, self).get_queryset(request)
        qs = qs.defer("content")
        return qs



# Generated by Django 4.2.1 on 2023-05-22 09:10

from django.db import migrations, models
import django.utils.timezone


class Migration(migrations.Migration):
    dependencies = [
        ("dbstorage", "0001_initial"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="dbfile",
            name="size",
        ),
        migrations.AddField(
            model_name="dbfile",
            name="accessed_on",
            field=models.DateTimeField(default=django.utils.timezone.now),
        ),
        migrations.AlterField(
            model_name="dbfile",
            name="id",
            field=models.BigAutoField(
                auto_created=True,
                primary_key=True,
                serialize=False,
                verbose_name="ID",
            ),
        ),
    ]
